part of '../vrchat_dart.dart';

extension Auth on VrchatDartImpl {
  /// Login to the VRChat API with a [username] and [password]
  /// 
  /// Logging in without a [username]/[password] will use the stored auth
  /// session if available
  Future<AuthResponse> login({
    String? username,
    String? password,
  }) async {
    try {
      final response = await rawApi.getAuthenticationApi().getCurrentUser(
        headers: {
          'Authorization':
              '${base64.encode(utf8.encode('$username:$password'))}'
        },
      );

      return AuthResponse(user: response.data);
    } on DioError catch (error) {
      final bodyJson = error.response?.data as Map<String, dynamic>;

      if (bodyJson['requiresTwoFactorAuth'] != null) {
        return AuthResponse(requiresTwoFactorAuth: true);
      } else {
        return AuthResponse(authError: true);
      }
    }
  }

  /// Verify a 2fa code
  ///
  /// Returns [true] on successful validation
  Future<bool> verify2fa(String code) async {
    try {
      await rawApi.getAuthenticationApi().verify2FA(
            inlineObject: InlineObject(
              (builder) => builder.code = code,
            ),
          );
    } on DioError {
      return false;
    }

    return true;
  }
}

class AuthResponse {
  /// True if this account requires two factor auth
  final bool requiresTwoFactorAuth;

  /// True if there was an error logging in
  final bool authError;

  /// The current user if authentication was successful
  final CurrentUser? user;

  /// Create an [AuthResonse]
  AuthResponse({
    this.requiresTwoFactorAuth = false,
    this.authError = false,
    this.user,
  });
}
